package org.processmining.framework.util.ui.widgets.traceview;

import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * Provides a filtered {@link ListModel} based on a supplied {@link ListModelFilter}.
 * 
 * @author F.Mannhardt
 *
 * @param <E>
 */
@SuppressWarnings("rawtypes")
public class FilteredListModel<T extends ListModel, E> extends AbstractListModel {

	private static final long serialVersionUID = -5708650645208821099L;

	public static interface ListModelFilter<E> {
		boolean accept(E e);
	}
	
	private ListModelFilter<E> currentFilter;
	private final T originalListModel;	
	private final List<Integer> visibleElements = new ArrayList<Integer>();

	public FilteredListModel(T source) {
		originalListModel = source;
		getOriginalListModel().addListDataListener(new ListDataListener() {
			public void intervalRemoved(ListDataEvent e) {
				boolean changed = doFilter();
				if (!changed) {
					fireIntervalRemoved(e.getSource(), e.getIndex0(), e.getIndex1());
				}
			}

			public void intervalAdded(ListDataEvent e) {
				boolean changed = doFilter();
				if (!changed) {
					fireIntervalAdded(e.getSource(), e.getIndex0(), e.getIndex1());
				}
			}

			public void contentsChanged(ListDataEvent e) {
				boolean changed = doFilter();
				if (!changed) {
					fireContentsChanged(e.getSource(), e.getIndex0(), e.getIndex1());
				}
			}
		});
	}

	public void setFilter(ListModelFilter<E> f) {
		currentFilter = f;
	}

	@SuppressWarnings("unchecked") // ListModel is unfortunately not generic in Java 1.6 and we rely on Hudson that uses 1.6
	public boolean doFilter() {		
		ListModelFilter<E> f = currentFilter;
		if (f != null) {
			visibleElements.clear();
			int count = getOriginalListModel().getSize();
			for (int i = 0; i < count; i++) {
				if (f.accept((E) getOriginalListModel().getElementAt(i))) {
					visibleElements.add(i);
				}
			}
			fireContentsChanged(this, 0, getSize() - 1);
			return true;
		}
		return false;
	}

	@Override
	public int getSize() {
		return (currentFilter != null) ? visibleElements.size() : getOriginalListModel().getSize();
	}

	@SuppressWarnings("unchecked") // ListModel is unfortunately not generic in Java 1.6 and we rely on Hudson that uses 1.6 
	@Override
	public E getElementAt(int index) {
		return (E) ((currentFilter != null) ? getOriginalListModel().getElementAt(visibleElements.get(index)) : getOriginalListModel().getElementAt(index));
	}

	public T getOriginalListModel() {
		return originalListModel;
	}

}